#include "TcpSocket.h"
#include "ShadeopUtils.h"


#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <sys/time.h>

#include <arpa/inet.h>
#include <unistd.h>
#include <netdb.h>
#include <sys/socket.h>
#include <sys/fcntl.h>
#include <errno.h>











TcpSocket::TcpSocket(int s)
	{
	sockfd=s;
	}

TcpSocket::TcpSocket(char *hostname,int port,bool listen_socket)
	{
	bzero((char *) &serv_addr,sizeof(serv_addr));
	serv_addr.sin_family=AF_INET;

	if(hostname!=NULL)
		{
		if((hostentry=gethostbyname(hostname))==NULL)
			ShadeopUtils::SystemError("No TCP IP address");
	
		sprintf(ipaddr,"%s",inet_ntoa(*((struct in_addr *)(*hostentry->h_addr_list))));
		
		serv_addr.sin_addr.s_addr=inet_addr(ipaddr);
		}
	else
		serv_addr.sin_addr.s_addr=INADDR_ANY;
		
	serv_addr.sin_port=htons(port);

	if((sockfd=socket(AF_INET,SOCK_STREAM,0))<0)
		ShadeopUtils::SystemError("No TCP IP socket");
		
	
	if(!listen_socket)
		{
		bool success=false;
		

		ShadeopUtils::Progress("Connecting");

		for(int i=0;i<100;++i)
			{
			if(connect(sockfd,(struct sockaddr *)&serv_addr,sizeof(serv_addr))>=0)
				{
				success=true;
				
				break;
				}
				
			// Rebuild the socket - seems to improve connection of later appearing destination
				
			close(sockfd);
			if((sockfd=socket(AF_INET,SOCK_STREAM,0))<0)
				ShadeopUtils::SystemError("No TCP IP socket");
			
			sleep(1);
			}
			
		if(!success)
			ShadeopUtils::SystemError("No TCP IP connect after 100 attempts");

		ShadeopUtils::Progress("Connected");
		}
	else
		{
		int v=1;                                      
		setsockopt(sockfd,SOL_SOCKET,SO_REUSEADDR,&v,sizeof(v)); 

		if(bind(sockfd,(struct sockaddr *)&serv_addr,sizeof(serv_addr))<0)
			ShadeopUtils::SystemError("No TCP IP bind");
		if(listen(sockfd,1024)<0)
			ShadeopUtils::SystemError("No TCP IP listen");
		}
	}
	
	
TcpSocket::TcpSocket(char *hostname,char *p,bool listen_socket)
	{
	struct servent *port=getservbyname(p,"tcp");
			
	if(port==NULL)
		ShadeopUtils::SystemError("Service not known"); //### (%s)",p);
	
	TcpSocket(hostname,port->s_port,listen_socket);
	}
	
	
	
TcpSocket::~TcpSocket()
	{
	close(sockfd);
	}

	
void TcpSocket::SelectNonBlockingInput()
	{
	fcntl(sockfd,F_SETFL,fcntl(sockfd,F_GETFL) | O_NONBLOCK);
	}
	
void TcpSocket::SelectBlockingInput()
	{
	fcntl(sockfd,F_SETFL,fcntl(sockfd,F_GETFL) & ~O_NONBLOCK);
	}
	
	
TcpSocket *TcpSocket::Accept()
	{
	int s;
	
	if((s=accept(sockfd,NULL,NULL))<0)
		{
		if(errno==EWOULDBLOCK)
			return(NULL);
		else
			ShadeopUtils::SystemError("No TCP IP accept");
		}
		
	TcpSocket *ts=new TcpSocket(s);

	return(ts);
	}	
	

int TcpSocket::HandleInputBytes(void *buffer,int n)
	{
	int r=recv(sockfd,buffer,n,0);
	
	if(r==-1)	
		{
		if(errno==EWOULDBLOCK)
			return(0);
		else
			ShadeopUtils::SystemError("Could not read from socket");
		}
		
//##	if(r==0)
//###		eod=true;
		
	return(r);
	}
	
	
void TcpSocket::HandleOutputBytes(const void *buffer,int n)
	{
	int r=send(sockfd,buffer,n,0);

	if(r==-1)	
		ShadeopUtils::SystemError("Could not write to socket");
	
	if(r!=n)
		ShadeopUtils::Error("Did not write required number of bytes to socket"); //###"+name+" (attempted %d, completed %d)",n,r);	
	}


//#### check this in CY	- does send always send as much as we need?



